/* NearestNeighborsSimilaritySubsets.java
 * Author: Nina Jeliazkova
 * Date: Feb 4, 2007 
 * Revision: 0.1 
 * 
 * Copyright (C) 2005-2006  Nina Jeliazkova
 * 
 * Contact: nina@acad.bg
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * All we ask is that proper credit is given for our work, which includes
 * - but is not limited to - adding the above copyright notice to the beginning
 * of your source code files, and to any copyright notice that you may distribute
 * with programs based on this work.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package chemCast.similarity.data;

import java.util.List;

import org.openscience.cdk.qsar.model.QSARModelException;

import ambit.data.molecule.IAtomContainersList;
import ambit.exceptions.AmbitException;
import ambit.similarity.ClassificationModelStatistics;
import ambit.similarity.DistanceComparable;
import ambit.similarity.IAttributeSelection;
import ambit.similarity.IClassificationModel;
import ambit.similarity.IDistanceFunction;
import ambit.similarity.IPredictionModel;
import ambit.similarity.NearestNeighborsSimilarity;
import ambit.similarity.PredictionModelStatistics;
import chemCast.application.StatusBar;
import chemCast.weka.ISubsetsModel;

public abstract class NearestNeighborsSimilaritySubsets<T> extends
        NearestNeighborsSimilarity<T> implements ISubsetsModel, IPredictionModel, IClassificationModel {
    
    /**
     * 
     */
    private static final long serialVersionUID = 4179492813969471175L;
    protected Subsets datasets;
    protected String predictedProperty;
    protected String classificationProperty;
    protected boolean prediction = true;


    public NearestNeighborsSimilaritySubsets(IDistanceFunction<T> distanceFunction, int knn) {
        super(distanceFunction, knn);
        setPredictedProperty("Predicted");
        setClassificationProperty("Group");


    }
	protected abstract Object readAttribute(T attributes, int attribute_index);
	protected abstract double readMaxAttribute(int attribute_index);
    protected abstract double readMinAttribute(int attribute_index);
	
    
    public Subsets getDatasets() {
        return datasets;
    }
    /**
     * Calls {@link #calculateComparableProperty(Object)}. Override for more efficient implementation.
     * @param list
     * @param index
     * @return
     * @throws AmbitException
     */
    protected abstract T readComparableProperty(IAtomContainersList list, int index) throws AmbitException ;
    protected abstract void writeComparableProperty(IAtomContainersList list, int index, T property) throws AmbitException ;
        //return calculateComparableProperty(list.getAtomContainer(index));
    
    
    
    public float predict(IAtomContainersList object1, int index1,
            IAtomContainersList object2, int index2) throws AmbitException {
        try {
            T t1 = transformAttributes(readComparableProperty(object1,index1));
            if (t1==null) t1 = transformAttributes(calculateComparableProperty(object1.getAtomContainer(index1)));
            
            T t2 = transformAttributes(readComparableProperty(object2,index2));
            if (t2==null) t2 = transformAttributes(calculateComparableProperty(object2.getAtomContainer(index2)));
            IDistanceFunction<T> df = getNearestNeighborsSearch().getDistanceFunction();
            //System.out.println(df.toString());
            return df.getDistance(t1,t2);
        } catch (Exception x) {
            throw new AmbitException(x);
        }
    }

    public void setDatasets(Subsets datasets) {
        this.datasets = datasets;

    }
    
    public String getResultField() {
    	if (datasets.getParentDataset() == null) return "result";
    	else
    		return  datasets.getParentDataset().getDatasetResultField();
    }

    @Override
    public void buildInitialize() throws AmbitException {
    	super.buildInitialize();
    }
    public void build() throws QSARModelException {
        try {
            buildInitialize();
        } catch (Exception x) {
            throw new QSARModelException(x.getMessage());
        }
        trainingPrefix = datasets.toString();
        datasets.firePropertyChange(StatusBar.Message_Status,null,"Build phase: " + toString() + " " + datasets.toString());

        String resultField = getResultField();

        int n = 0;
        for (int s=0; s < datasets.size();s++)
            if (datasets.get(s).isEnabled()) n++;
        
        prediction = (n==1); 
        
        if (n==0) throw new QSARModelException("No groups defined for  classification. Please use manu <Group/New group> to assign chemicals to groups.");
                
        for (int s=0; s < datasets.size();s++)
            
            if (datasets.get(s).isEnabled()) 
            try {
                
                IAtomContainersList list = datasets.get(s).getData();
                if (list == null) continue;
                for (int i=0; i < list.getAtomContainerCount();i++) 
                    try {
                        //System.out.println(i);
                        
                        datasets.firePropertyChange(StatusBar.Message_Progress,null,new Double(100*i/list.getAtomContainerCount()));
                        
                        T t = readComparableProperty(list, i);
                        if (t == null) t = calculateComparableProperty(list.getAtomContainer(i));
                        
                        if (t!=null) {
                            Object property = null;
                            if (prediction) property = list.getProperty(i, resultField);
                            else property = datasets.get(s).toString();
                            getNearestNeighborsSearch().addObject(t,property);
                            //System.out.println(property);
                            writeComparableProperty(list, i, t);
                        }
                        
                        
                    } catch (Exception x) {
                        x.printStackTrace();
                    }
                    
            } catch (Exception x) {
                datasets.firePropertyChange(StatusBar.Message_Error,null,x);
            }
 
       double[] attribute_weights = null;
       if (attributeSelection != null) {     
    	   attributeSelection.evaluateAttributes(getNearestNeighborsSearch(),getNumberOfAttributes());
    	   attributeSelection.transformAttributes(getNearestNeighborsSearch());
       } else
    	   attribute_weights = null;
       /*
       if (attribute_weights != null) 
    	   for (int i=0; i < attribute_weights.length;i++)
    		   if (attribute_weights[i] > 0) 
    			   System.out.println(i + " " + attribute_weights[i]);
    			   */
       setPredicting(true);     
       setDatasets(null);

    }
    
    protected abstract int getNumberOfAttributes();


    public void predict() throws QSARModelException {
        //histogram.clear();
        if (modelStatistics != null) {
        	modelStatistics.clear(datasets.toString());
        	modelStatistics.setTitle(getSimilarityProperty());
        }
        datasets.firePropertyChange(StatusBar.Message_Status,null,"Prediction phase: " + toString() + " " + datasets.toString());
        String sp = trainingPrefix + "." + getSimilarityProperty();
        String pp = sp + "." + getPredictedProperty();
        String cp = sp + "." + getClassificationProperty();
        for (int s=0; s < datasets.size();s++)
            if (datasets.get(s).isEnabled()) 
            try {
                IAtomContainersList list = datasets.get(s).getData();
                if (list == null) continue;
                for (int i=0; i < list.getAtomContainerCount();i++) 
                    try {
                        //System.out.println(i);
                        datasets.firePropertyChange(StatusBar.Message_Progress,null,new Double(100*i/list.getAtomContainerCount()));
                        T query = transformAttributes(readComparableProperty(list, i));
                        if (query == null) 
                            query = transformAttributes(calculateComparableProperty(list.getAtomContainer(i)));
                        if (query != null) { 
                            List<DistanceComparable<T>> neighbors = nearestNeighborsSearch.getNearestNeighbors(query, kNN);
                            Double distance = new Double(calculateAverageDistance(query,neighbors));
                            
                            list.setProperty(i, sp,distance);
                            //histogram.update(distance);
                            if (prediction) {
                                if (modelStatistics == null) modelStatistics = new PredictionModelStatistics(getSimilarityProperty());
                                Object property = calculateProperty(query, neighbors);
                                list.setProperty(i, pp,property);
                                //TODO training - test
                                modelStatistics.addPoint(datasets,list.getProperty(i, getPredictedProperty()), property);
                                modelStatistics.addValue(datasets,datasets.get(s).toString(), distance);
                            } else {
                            	String title = getSimilarityProperty()+"."+getClassificationProperty();
                                if (modelStatistics == null) modelStatistics = new ClassificationModelStatistics(title);
                                else modelStatistics.setTitle(title);
                                Object property = classifyQuery(query, neighbors);
                                list.setProperty(i, cp,property);
                                modelStatistics.addPoint(datasets,datasets.get(s).toString(), property);
                                modelStatistics.addValue(datasets,datasets.get(s).toString(), distance);
                            }
                        } else
                            list.setProperty(i, sp, null);
                        
                    } catch (Exception x) {
                        datasets.firePropertyChange(StatusBar.Message_Error,null,x);
                        list.setProperty(i, sp, null);
                        list.setProperty(i, pp, null);
                    }
                    
            } catch (Exception x) {
                x.printStackTrace();
            }
         setDatasets(null);
        
    }


    public String[] getProperties() {
        if (prediction)
        return new String[] {
                getTrainingPrefix() + "." + getSimilarityProperty(),
                getTrainingPrefix() + "." + getSimilarityProperty() + "." + getPredictedProperty()
                };
        else
            return new String[] {
                getTrainingPrefix() + "." + getSimilarityProperty(),
                getTrainingPrefix() + "." + getSimilarityProperty() + "." + getClassificationProperty()
                };
            
    }
    public void setParameter(Object parameter, Object value) {
        super.setParameter(parameter, value);
        if (parameter.equals(p_classification_property)) classificationProperty = value.toString();
        else if (parameter.equals(p_predicted_property)) predictedProperty = value.toString();  

    }
    public Object getParameter(Object parameter) {
        Object p = super.getParameter(parameter);
        if (p!=null) return p;
        else if (p_classification_property.equals(parameter)) return getClassificationProperty();
        else if (p_predicted_property.equals(parameter)) return getPredictedProperty();
        else return null;
    }
    public synchronized String getPredictedProperty() {
        return predictedProperty;
    }

    public synchronized void setPredictedProperty(String predictedProperty) {
        this.predictedProperty = predictedProperty;
    }

    public synchronized String getClassificationProperty() {
        return classificationProperty;
    }

    public synchronized void setClassificationProperty(String classificationProperty) {
        this.classificationProperty = classificationProperty;
    }


	public IAttributeSelection<T> getAttributeSelection() {
		return attributeSelection;
	}


	public void setAttributeSelection(IAttributeSelection<T> attributeSelection) {
		this.attributeSelection = attributeSelection;
	}

}
