/*
Copyright Ideaconsult Ltd. (C) 2005-2007 

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
package toxTree.tree;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.Observable;

import org.openscience.cdk.interfaces.IAtomContainer;

import toxTree.core.IDecisionCategory;
import toxTree.core.IDecisionMethod;
import toxTree.core.IDecisionResult;
import toxTree.core.IDecisionRule;
import toxTree.exceptions.DMethodNotAssigned;
import toxTree.exceptions.DecisionMethodException;
import toxTree.exceptions.DecisionResultException;
import toxTree.logging.TTLogger;

/**
 * A default class implementing {@link toxTree.core.IDecisionResult}
 * @author Nina Jeliazkova nina@acad.bg<br>
 * @version 0.1, 2005-4-30
 */
public class TreeResult extends Observable implements IDecisionResult {
    protected static transient TTLogger logger = null;
    /**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = 87786882555929168L;
    protected transient PropertyChangeSupport changes = null;    
	protected IDecisionMethod decisionMethod = null;
	protected IDecisionCategory category = null;
	
	protected ArrayList<RuleResult> ruleResults = null;
	
	protected transient ProgressStatus status;
	
	public static final transient String _eMethodNotAssigned = "Decision method not assigned!";
	public static final transient String _eResultNotEstimated = "Decision method has not been applied yet!";	
	public static final transient String _eResultIsEstimating = "Decision method is being applied now ...!";	
	public static final transient String _eUnknownState = "Unknown state !\t";
	public static final transient String _eError = "ERROR: \t";		
		
	public static final transient String _pClassID = "ClassID";
	public static final transient String _pDecisionMethod = "DecisionMethod";
	public static final transient String _pRuleResult = "RuleResult";
	
	/**
	 * 
	 * Constructor
	 * @param method
	 */
	public TreeResult() {
		super();
		status = new ProgressStatus();
		ruleResults = new ArrayList<RuleResult>();
		if (logger ==null) logger = new TTLogger(TreeResult.class);
	}
	public void clear() {

		clearResults();
	}
	
	protected void clearResults() {
	    firePropertyChangeEvent(_pClassID,category,null);
	    category = null;	    
	    if (ruleResults != null) {
	        ruleResults.clear();
	    }
	    status.clear();
		setChanged();
		notifyObservers();	    
	}
	public boolean isError() {
		return status.isError();
	}
	public void setError(String message) {
		status.setError(message);
		setChanged();
    	notifyObservers();
	}
	/* (non-Javadoc)
     * @see toxTree.core.IDecisionResult#isEstimated()
     */
    public boolean isEstimated() {
        return status.isEstimated();
    }
    /* (non-Javadoc)
     * @see toxTree.core.IDecisionResult#isEstimating()
     */
    public boolean isEstimating() {
        return status.isEstimating();	
    }
    /* (non-Javadoc)
     * @see toxTree.core.IDecisionResult#setEstimating()
     */
    public void setEstimating() {
        status.setEstimating();
		setChanged();
		notifyObservers();       
    }
    public void setEstimated() {
    	status.setEstimated();
    	setChanged();
    	notifyObservers();
    	
    }
    /* (non-Javadoc)
     * @see toxTree.core.IDecisionResult#setEstimated(boolean)
     */
    public void setEstimated(boolean value) {
    	status.setEstimated(value);
		setChanged();
		notifyObservers();
    }
	public IDecisionCategory getCategory() {
		return category;
	}
	public void setCategory(IDecisionCategory category) {
	    firePropertyChangeEvent(_pClassID,this.category,category);
	    if ((this.category == null) || (this.category.compareTo(category) < 0)) {
	    	logger.info("Assign category\t",category);
	    	this.category = category;
	    }
		((RuleResult) ruleResults.get(ruleResults.size()-1)).setCategory(category);
		setChanged();
		notifyObservers();		
	}
	/* (non-Javadoc)
	 * @see toxTree.core.IDecisionResult#setMolecule(org.openscience.cdk.AtomContainer)
	 */
	public void setMolecule(IAtomContainer molecule) {
		((RuleResult) ruleResults.get(ruleResults.size()-1)).setMolecule(molecule);
		setChanged();
		notifyObservers();		
	}
	public void setSilent(boolean silent) throws DecisionResultException {
		((RuleResult) ruleResults.get(ruleResults.size()-1)).setSilent(silent);
		setChanged();
		notifyObservers();			
	}
	/*
	public void addRuleResult(int ruleIndex, boolean value)
			throws DecisionResultException {
		if (decisionMethod == null) 
		    throw new DMethodNotAssigned(_eMethodNotAssigned);
		IDecisionRule rule = decisionMethod.getRule(ruleIndex);
		if (rule == null)
			 throw new DecisionResultException("Rule #" + ruleIndex + " not assigned!");
		results.add(new RuleResult(rule,value));
	}
	*/
	public void addRuleResult(IDecisionRule rule, boolean value,IAtomContainer molecule) throws DecisionResultException {
		if (decisionMethod == null)   throw new DMethodNotAssigned(_eMethodNotAssigned);
		if (rule == null) throw new DecisionResultException("Rule # not assigned!");
		ruleResults.add(new RuleResult(rule,value));
		setMolecule(molecule);
	}
	/**
	 * return StringBuffer with textual explanation of the rules executed
	 */
	public StringBuffer explain(boolean verbose) throws DecisionResultException {
		try {
		    StringBuffer b;
		    switch (status.estimated) {
		    case 2: {
		    	b = new StringBuffer();
		    	IAtomContainer ac = null;
		    	boolean first = true;
		    	String delimiter = "";
		    	for (int i=0;i < ruleResults.size();i++) {
		    		if ((!verbose) && (i>0)) delimiter = ",";
		    		else delimiter = "";
		    		RuleResult r = ((RuleResult)ruleResults.get(i));
		    		if (r.isSilent()) continue;
		    		IAtomContainer mol = r.getMolecule();
		    		if (ac == null) ac = mol;
		    		if ((ac != null) && (ac.getID() != mol.getID())) {
		    			if (!first)  if (!verbose) b.append(")"); else ;
		    			else first = false;
		    			if (verbose) b.append("\n"); else  b.append("(");
		    			ac = mol;
		    		} else b.append(delimiter);
		    		
		    		b.append(r.explain(verbose));
		    		if (verbose) b.append("\n");
		    	}
	    		if (!first) if (!verbose) b.append(")");		    	
				return b;
		    }
		    case 1: {
		        b = new StringBuffer();
		        b.append(_eResultIsEstimating);
		        return b;
		    } 
		    case 0: {
		        b = new StringBuffer();
		        b.append(_eResultNotEstimated);
		        return b;
		    }
		    case 4: {
		        b = new StringBuffer();
		        b.append(_eError+status.getErrMessage());
		        return b;
		    }		    
		    default: {
		        b = new StringBuffer();
		        b.append(_eUnknownState);
		        return b;		        
		    }
		    }
		//} catch (DecisionMethodException x) {
			//throw new DecisionResultException(x);
		} catch (NullPointerException x) {
			throw new DMethodNotAssigned(_eMethodNotAssigned);
		}
	}	
	/**
	 * toString
	 */
	@Override
	public String toString() {
	    if (decisionMethod == null) return _eMethodNotAssigned;
	    switch (status.estimated) {
	    case 0: return _eResultNotEstimated;
	    case 1: return _eResultIsEstimating;
	    case 2: return decisionMethod.toString() + "\t Class\t" + category.toString();
	    case 4: return "Error\t"+status.getErrMessage();	    
	    default: return _eUnknownState;
	    }
	}

	/**
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(Object o) {
		TreeResult r = (TreeResult) o;
		return category.compareTo(r.getCategory());
	}
	
	public IDecisionMethod getDecisionMethod() {
		return decisionMethod;
	}
	public void setDecisionMethod(IDecisionMethod decisionMethod) {
		if (decisionMethod == null) {
		    if (this.decisionMethod != null) clear();
		    firePropertyChangeEvent(_pDecisionMethod,this.decisionMethod,decisionMethod);		    
		    this.decisionMethod = null;
	    
		} else {
		    if ((this.decisionMethod != null) &&
		        (this.decisionMethod.equals(decisionMethod))) 
		        return;
		    
		    firePropertyChangeEvent(_pDecisionMethod,this.decisionMethod,decisionMethod);	    
			this.decisionMethod = decisionMethod;
			ruleResults.clear();

		}
		setChanged();
		notifyObservers();		
	}

	public boolean classify(IAtomContainer mol) throws DecisionResultException {
		try {
		    boolean r  = decisionMethod.classify(mol,this);
		    setEstimated(true);
			//Toolkit.getDefaultToolkit().beep();
			return r;
		} catch (DecisionMethodException x) {
			//x.printStackTrace();
			setEstimated(false);
			throw new DecisionResultException(x);
		} catch (NullPointerException x) {
			//x.printStackTrace();
			setEstimated(false);
			if (decisionMethod == null)
				throw new DMethodNotAssigned(x);
			else throw new DecisionResultException(x);
		}
	}
	public void addPropertyChangeListener(PropertyChangeListener l) {
	    if (changes == null) changes = new PropertyChangeSupport(this);
        changes.addPropertyChangeListener(l);
	}
	public void removePropertyChangeListener(PropertyChangeListener l) {
	    if (changes != null)
	        changes.removePropertyChangeListener(l);	    
	}
	protected void fireChangeEvent(PropertyChangeEvent e) {
	    if (changes != null) 
	        changes.firePropertyChange(e);
	}
	protected void firePropertyChangeEvent(String propertyName, int oldValue, int newValue) {
	    if (changes != null) 
	        changes.firePropertyChange(propertyName, oldValue, newValue);
	}
	protected void firePropertyChangeEvent(String propertyName, boolean oldValue, boolean newValue) {
	    if (changes != null) 
	        changes.firePropertyChange(propertyName, oldValue, newValue);
	}
	
	protected void firePropertyChangeEvent(String propertyName, Object oldValue, Object newValue) {
	    if (changes != null) 
	        changes.firePropertyChange(propertyName, oldValue, newValue);
	}

    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
	public boolean equals(Object obj) {
        IDecisionResult r = (IDecisionResult) obj;
        //if both methods are unassigned assume equal
        if ((decisionMethod == null) && (r.getDecisionMethod() == null)) return true;
        //if one is unassigned but the other is, return false
        if ((decisionMethod == null) || (r.getDecisionMethod() == null)) return false;
        
        if ((category.equals(r.getCategory())) &&  
        	(decisionMethod.equals(r.getDecisionMethod()))) {
            int nr = getRuleResultsCount();
            if (nr != r.getRuleResultsCount()) return false;
            try {
	            for (int i =0; i < nr; i++)
	                if (!getRuleResult(i).equals(r.getRuleResult(i))) {
	                	System.out.println(getRuleResult(i)+"\tdoesn't match\t"+r.getRuleResult(i));
	                	return false;
	                }
	            return true;    
            } catch (DecisionResultException x) {
            	x.printStackTrace();
                return false;
            }
            
        } else return false;
    }
    /* (non-Javadoc)
	 * @see toxTree.core.IDecisionResult#getRuleResultsCount()
	 */
	public int getRuleResultsCount() {
		return ruleResults.size();
	}
	/* (non-Javadoc)
	 * @see toxTree.core.IDecisionResult#getMolecule(int)
	 */
	public IAtomContainer getMolecule(int index) throws DecisionResultException {
		try {
			return ((RuleResult) ruleResults.get(index)).getMolecule();
		} catch (Exception x) {
			throw new DecisionResultException(x);
		}
		
	}
	/* (non-Javadoc)
	 * @see toxTree.core.IDecisionResult#getCategory(int)
	 */
	public IDecisionCategory getCategory(int index) throws DecisionResultException {
		try {
			return ((RuleResult) ruleResults.get(index)).getCategory();
		} catch (Exception x) {
			throw new DecisionResultException(x);
		}
	}
	/* (non-Javadoc)
	 * @see toxTree.core.IDecisionResult#getRuleResult(int)
	 */
	public RuleResult getRuleResult(int index) throws DecisionResultException {
		try {
			return (RuleResult) ruleResults.get(index);
		} catch (Exception x) {
			throw new DecisionResultException(x);
		}
	}
	/* (non-Javadoc)
	 * @see toxTree.core.IDecisionResult#getRule(int)
	 */
	public IDecisionRule getRule(int index) throws DecisionResultException {
		try {
			return ((RuleResult) ruleResults.get(index)).getRule();
		} catch (Exception x) {
			throw new DecisionResultException(x);
		}
	}
	public void assignResult(IAtomContainer mol) throws DecisionResultException {
        mol.setProperty(
        		getResultPropertyName(),
                getCategory().toString());
        mol.setProperty(
        		getResultPropertyName()+"#explanation",
                explain(false).toString());
        /*
            mol.setProperty(
                    getDecisionMethod().getClass().getName(),
                    getCategory().toString());
            mol.setProperty(
                    getClass().getName(),
                    explain(false).toString());
	    
        */
	}
	public String getResultPropertyName() {
		if (getDecisionMethod() == null) return "Unassigned method";
		else return getDecisionMethod().getTitle();
	}
	public ArrayList<RuleResult> getRuleResults() {
		return ruleResults;
	}
	public void setRuleResults(ArrayList<RuleResult> ruleResults) {
		this.ruleResults = ruleResults;
	}
    public int getPriority() {
        IDecisionMethod m = getDecisionMethod();
        if (m ==null) return Integer.MAX_VALUE;
        else return m.getPriority();
    }
    public void setPriority(int priority) {
        IDecisionMethod m = getDecisionMethod();
        if (m ==null) return;
        else m.setPriority(priority);
        
    }
	
}
